<?php

namespace AltSolution\Admin\Http\Controllers\Auth;

use AltSolution\Admin\Auth\TwoFactorAuthInterface;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Validation\ValidationException;
use Illuminate\Auth\Events as LaravelEvents;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\Facades\Auth;

class LoginController extends Controller
{
    public function __construct()
    {
        $this->middleware('guest')->except('logout');
    }

    public function login()
    {
        $this->layout->setTitle(trans('admin::auth.sign_title'));
        return view('admin::auth.login');
    }

    // NOTE: not forget update this method on updating laravel version
    public function postLogin(Request $request, TwoFactorAuthInterface $f2a)
    {
        $request->validate([
            'email' => 'required|string|email',
            'password' => 'required|string',
        ]);

        $throttleKey = $this->throttleKey($request);
        if (RateLimiter::tooManyAttempts($throttleKey, 5)) {
            event(new LaravelEvents\Lockout($request));

            $seconds = RateLimiter::availableIn($throttleKey);
            throw ValidationException::withMessages([
                'email' => trans('auth.throttle', [
                    'seconds' => $seconds,
                    'minutes' => ceil($seconds / 60),
                ]),
            ]);
        }

        $guard = $this->guard();
        $credentials = $request->only('email', 'password');

        if ($guard->once($credentials)) {
            RateLimiter::clear($throttleKey);

            $remember = $request->filled('remember');
            $user = $guard->getLastAttempted();

            $is2FAEnabled = config('admin.auth_2fa');
            if ($is2FAEnabled && $f2a->isEnabledFor($user)) {
                $f2a->putUser($user, $remember);
                try {
                    $f2a->sendVerification();
                    return redirect()->route('admin::2fa');
                } catch (\Throwable $e) {
                    logger()->error('Failed sent verification code for 2FA. ' . $e->getMessage());
                    $f2a->forgetUser();
                    return redirect()->back()
                        ->withInput($request->only('email', 'remember'))
                        ->withErrors([
                            'email' => trans('admin::auth.f2a_sent_failed'),
                        ]);
                }
            }

            $guard->login($user, $remember);
            $request->session()->regenerate();

            return redirect()->intended($this->redirectPath());
        }

        /** @see \Illuminate\Auth\SessionGuard::fireFailedEvent */
        $user = $guard->getLastAttempted();
        // TODO: weak
        $guardName = Auth::getDefaultDriver();
        event(new LaravelEvents\Failed($guardName, $user, $credentials));

        RateLimiter::hit($throttleKey);

        throw ValidationException::withMessages([
            'email' => trans('auth.failed'),
        ]);
    }

    public function logout(Request $request)
    {
        $this->guard()->logout();
        $request->session()->invalidate();
        $request->session()->regenerateToken();

        return redirect($this->redirectPath());
    }

    /**
     * Get the guard to be used during authentication.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return Auth::guard();
    }

    /**
     * Get the post register / login redirect path.
     *
     * @return string
     */
    public function redirectPath()
    {
        return '/' . cms_admin_url();
    }

    /**
     * Get the rate limiting throttle key for the request.
     *
     * @return string
     */
    public function throttleKey(Request $request)
    {
        return Str::lower($request->input('email')) . '|' . $request->ip();
    }
}
