<?php

namespace AltSolution\Admin\Http\Controllers\Auth;

use AltSolution\Admin\Auth\Notifications\ResetPassword as ResetPasswordNotification;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Password;

class PasswordController extends Controller
{
    public function __construct()
    {
        $this->middleware('guest');
    }

    public function forgot()
    {
        $this->layout->setTitle(trans('admin::auth.reset_title'));
        return view('admin::auth.passwords.email');
    }

    public function postForgot(Request $request)
    {
        $this->validate($request, ['email' => 'required|email']);

        // We will send the password reset link to this user. Once we have attempted
        // to send the link, we will examine the response then see the message we
        // need to show to the user. Finally, we'll send out a proper response.
        $credentials = $request->only('email');
        $response = $this->broker()->sendResetLink($credentials, function ($user, $token) {
            $user->notify(new ResetPasswordNotification($token));
        });

        // TODO: translate
        if ($response == Password::RESET_LINK_SENT) {
            return back()->with('status', trans($response));
        } else {
            return back()
                ->withInput($credentials)
                ->withErrors(['email' => trans($response)]);
        }
    }

    public function reset(Request $request, $token = null)
    {
        $this->layout->setTitle(trans('admin::auth.reset_title'));
        return view('admin::auth.passwords.reset')->with(
            ['token' => $token, 'email' => $request->email]
        );
    }

    public function postReset(Request $request)
    {
        $request->validate([
            'token' => 'required',
            'email' => 'required|email',
            'password' => 'required|confirmed|min:8',
        ]);

        $credentials = $request->only('email', 'password', 'password_confirmation', 'token');
        // Here we will attempt to reset the user's password. If it is successful we
        // will update the password on an actual user model and persist it to the
        // database. Otherwise we will parse the error and return the response.
        $response = $this->broker()->reset($credentials, function ($user, $password) {
            $user->password = Hash::make($password);
            $user->setRememberToken(Str::random(60));
            $user->save();
            event(new PasswordReset($user));
            $this->guard()->login($user);
        });

        // If the password was successfully reset, we will redirect the user back to
        // the application's home authenticated view. If there is an error we can
        // redirect them back to where they came from with their error message.
        if ($response == Password::PASSWORD_RESET) {
            $this->layout->addNotify('success', trans($response));
            return redirect($this->redirectPath());
            //->with('status', trans($response));
        } else {
            return redirect()->back()
                ->withInput($request->only('email'))
                ->withErrors(['email' => trans($response)]);
        }
    }

    /**
     * Get the broker to be used during password reset.
     *
     * @return \Illuminate\Contracts\Auth\PasswordBroker
     */
    protected function broker()
    {
        return Password::broker();
    }

    /**
     * Get the guard to be used during authentication.
     *
     * @return \Illuminate\Contracts\Auth\StatefulGuard
     */
    protected function guard()
    {
        return Auth::guard();
    }

    /**
     * Get the post register / login redirect path.
     *
     * @return string
     */
    public function redirectPath()
    {
        return '/' . cms_admin_url();
    }
}
