<?php

namespace AltSolution\Admin\Upload;

use Exception;
use Illuminate\Http\UploadedFile;
use Illuminate\Support\Arr;
use SplFileInfo;

class UploadManager implements UploadManagerInterface
{
    /**
     * Store uploads in this directory
     * @var string
     */
    protected $directory;
    /**
     * Prefix for generating url
     * @var string
     */
    protected $url;
    /**
     * Permission for creating directories
     * @var int
     */
    protected $permission;

    public function __construct(array $options)
    {
        $this->directory = Arr::get($options, 'directory');
        if (empty($this->directory)) {
            throw new Exception('Upload directory must be not empty');
        }
        $this->url = Arr::get($options, 'url');
        if (empty($this->url)) {
            $this->url = '/' . $this->directory;
        }
        $this->permission = Arr::get($options, 'permission', 0777);
    }

    public function storeUploadedFile(UploadedFile $file, $directory)
    {
        $fileName = $this->newFileName($file->getClientOriginalName());

        $dir = $this->getDir($directory, $fileName);
        if (!is_dir($dir)) {
            mkdir($dir, $this->permission, true);
        }

        $originalPath = $dir . $fileName;
        copy($file->getRealPath(), $originalPath);

        return $fileName;
    }

    public function storeFile(SplFileInfo $file, $directory)
    {
        if (!$file->isFile()) {
            throw new Exception('Invalid file passed as parameter: ' . $file);
        }
        $fileName = $this->newFileName($file->getFilename());

        $dir = $this->getDir($directory, $fileName);
        if (!is_dir($dir)) {
            mkdir($dir, $this->permission, true);
        }

        $originalPath = $dir . $fileName;
        copy($file->getRealPath(), $originalPath);

        return $fileName;
    }

    public function storeUrl($url, $directory)
    {
        $extension = pathinfo($url, PATHINFO_EXTENSION);
        $tempFile = tempnam(sys_get_temp_dir(), 'file') . '.' . $extension;
        $content = file_get_contents($url);
        if ($content === false) {
            throw new Exception('Failed fetch url: ' . $url);
        }
        file_put_contents($tempFile, $content);
        $file = new SplFileInfo($tempFile);

        $fileName = $this->storeFile($file, $directory);

        @unlink($tempFile);
        //register_shutdown_function('unlink', $tempFile);

        return $fileName;
    }

    public function storeContent($filename, $content, $directory)
    {
        $extension = pathinfo($filename, PATHINFO_EXTENSION);
        $tempFile = tempnam(sys_get_temp_dir(), 'file') . '.' . $extension;
        file_put_contents($tempFile, $content);
        $file = new SplFileInfo($tempFile);

        $fileName = $this->storeFile($file, $directory);

        @unlink($tempFile);

        return $fileName;
    }

    public function delete($fileName, $directory)
    {
        $dir = $this->getDir($directory, $fileName);
        if (is_file($dir . $fileName)) {
            unlink($dir . $fileName);
        }
    }

    public function generateUrl($fileName, $directory)
    {
        if (!$fileName) {
            return null;
        }

        $baseUrl = $this->getUrl($directory, $fileName);

        return $baseUrl . $fileName;
    }

    public function retrieveFile($fileName, $directory)
    {
        if (!$fileName) {
            return null;
        }

        $dir = $this->getDir($directory, $fileName);

        return $dir . $fileName;
    }

    protected function getDir($dir, $fileName)
    {
        $dirSuffix = null;
        if ($fileName) {
            $hash = sha1($fileName);
            $dirSuffix = substr($hash, 0, 2) . '/' . substr($hash, 2, 2) . '/';
        }

        $fullDir = $this->directory . '/' . $dir . '/' . $dirSuffix;
        // TODO: allow absolute directory
        $fullDir = public_path($fullDir);

        return $fullDir;
    }

    protected function getUrl($dir, $fileName)
    {
        $dirSuffix = null;
        if ($fileName) {
            $hash = sha1($fileName);
            $dirSuffix = substr($hash, 0, 2) . '/' . substr($hash, 2, 2) . '/';
        }

        $url = $this->url . '/' . $dir . '/' . $dirSuffix;

        return $url;
    }

    protected function newFileName($oldFileName)
    {
        $ts = (int)(microtime(true) * 1e4);
        $tsPart = rtrim(base_convert($ts, 10, 32), '=');
        $fn = sha1($oldFileName);
        $fnPart = substr($fn, rand(0, 32), 8);
        $pathPart = parse_url($oldFileName, PHP_URL_PATH);
        $ext = strtolower(pathinfo($pathPart, PATHINFO_EXTENSION));

        return sprintf('%s_%s.%s', $tsPart, $fnPart, $ext);
    }
}
